function [spec,fspec,tspec] = computespecgrams(signalred,fs,wndw,step,frng)

% Extend signal by mirroring if only a short piece is missing for a full
% window.
last = mod(size(signalred,2),wndw);
if last <= wndw/2 && last > 0
    signalred(:,end+(1:last)) = signalred(:,end-1:-1:end-last);
end

% Compute spectogram of ECG.
[spec,fspec,tspec] = periodograms(signalred(1,:),wndw,fs,[],step);
idx = frng(1) <= fspec & fspec <= frng(2);
fspec = fspec(idx);
spec = permute(spec(idx,:),[3 2 1]);
spec(2:3,:,:) = nan;

% Include spectogram of BP/PPG if possible.
if all(isfinite(signalred(2,:)))
    tmp = periodograms(signalred(2,:),wndw,fs,[],step);
    spec(2,:,:) = permute(tmp(idx,:),[3 2 1]);
end

% Include spectogram of SV if possible.
if all(isfinite(signalred(3,:)))
    tmp = periodograms(signalred(3,:),wndw,fs,[],step);
    spec(3,:,:) = permute(tmp(idx,:),[3 2 1]);
end

% Take absolute value.
spec = abs(spec);

% Normalize frequency and interpret decoupled sensors as missing data.
specsum = sum(spec,3);
specsum(specsum/size(spec,3) < 1e-6) = 0;
spec = bsxfun(@rdivide,spec,specsum);
spec(~isfinite(spec)) = nan;

end
