function [X,Y,tspec] = features(B,sig,lookback,lookahead,tspec,tann,options)
%FEATURES Computes features given factor matrix B. Also computes labels for
%training.
%
%   X = features(B,spec)
%
%   X = features(B,spec,lookback,lookahead) the number of PSD windows to
%   look back and ahead at.
%
%   [X,Y] = features(B,spec,lookback,lookahead,tspec,tann) computes labels
%   Y for each row of X based on the annotations in tann.

if nargin < 2 || isempty(lookback)
    lookback = 10;
end
if nargin < 3 || isempty(lookahead)
    lookahead = 10;
end
if nargin < 6
    options.balanced = false;
    options.idealfeatures = false;
end

% Parameters.
fs = 250;
wndw = 75;
step = 10;
B = B.';

% % Extend sig if necessary (similar to computespecgrams).
% sig = abs(sig);
% last = mod(size(sig,2),wndw);
% if last <= wndw/2 && last > 0
%     sig(:,end+(1:last)) = sig(:,end-1:-1:end-last);
% end
% 
% % Add amplitude features to B.
% p = [25 50 75 100];
% for i = 1:1 % Only ECG is guaranteed.
%     tmp = sig(i,:);
%     n = length(tmp);
%     idx = bsxfun(@plus,(1:wndw)',step*(0:floor((n-wndw)/step)));
%     idx(idx > n) = n;
%     tmp = tmp(idx);
%     for j = 1:length(p)
%         B(end+1,:) = prctile(tmp,p(j),1);
%     end
% end

% Convert B to feature matrix X.
width = lookback+1+lookahead;
X = B(:,permute(bsxfun(@plus,(1:size(B,2)-width+1)'-1,1:width),[3 2 1]));
X = reshape(X,width*size(B,1),[])';

% Compute labels Y.
if nargout > 1 && nargin > 4
    
    tspec = tspec(lookback+1:end-lookahead);
    if options.idealfeatures
        % Keep only "ideal" training samples.
        rerr = 10;
        Ycert = any(bsxfun(@le,tspec(:)*fs,tann(:)'*fs-rerr) & ...
                bsxfun(@ge,tspec(:)*fs+wndw,tann(:)'*fs+rerr),2);
        Ynear = any(bsxfun(@le,tann(:)'*fs,tspec(:)*fs+wndw+rerr/2) & ...
                bsxfun(@ge,tann(:)'*fs,tspec(:)*fs-rerr/2),2);
        del = Ynear & ~Ycert;
        Y = Ycert(~del);
        X = X(~del,:);
    else
        % Return all training samples.
        rerr = 2;
        Y = any(bsxfun(@le,tspec(:)*fs,tann(:)'*fs-rerr) & ...
                bsxfun(@ge,tspec(:)*fs+wndw,tann(:)'*fs+rerr),2);
    end
    
    if options.balanced
        % Choose random subset so that P/N = 1.
        pos = find(Y);
        neg = find(~Y);
        idx = [pos;neg(randi(length(neg),length(pos),1))];
        Y = Y(idx);
        X = X(idx,:);
    end
    
    % Make labels -1 and 1.
    Y = Y*2-1;
    
elseif nargout > 1
    
    tspec = tspec(lookback+1:end-lookahead);
    Y = [];

end

end
