function [A,B,C] = nntf(X,k,options)
% Nonnegative tensor factorization X = cpdgen({A,B,C})

% Set options.
if nargin<2, k = 3; end
if nargin<3, options=struct; end
if ~isfield(options,'MaxIter'), options.MaxIter = 50; end
if ~isfield(options,'Display'), options.Display = 1; end
epsilon = 100*eps;

% Format tensor.
W=isnan(X);
X(W)=0;
totfin = sum(~W(:));

% Initialize.
[r,c,t]=size(X);
if isfield(options,'A0') A = options.A0; k = size(A,2);
else A = rand(r,k);
end
A = max(A,eps);
if isfield(options,'B0'), B = options.B0; k = size(B,2);
else B = rand(c,k);
end
B = max(B,eps);
if isfield(options,'C0') C = options.C0; k = size(C,2);
else C = rand(t,k);
end
C = max(C,eps);

% Test to see if only B should be updated.
onlyupdateB = isfield(options,'A0') && isfield(options,'C0');

% If only B should be updated, try a good initialization.
if onlyupdateB && ~isfield(options,'B0')
    B = tens2mat(X,2)*pinv(kr(C,A).');
    B = max(B,eps);
end

for i=1:options.MaxIter
    
    % Update factor matrices.        
    if ~onlyupdateB
        Xhat = cpdgen({A,B,C});
        Xhat(W) = 0;
        %A = A .* ( tens2mat(X,1)*kr(C,B) ) ./ max( tens2mat(Xhat,1).*kr(C,B) , epsilon );
        A = A .* mtkrprod(X,{A,B,C},1) ./ max( mtkrprod(Xhat,{A,B,C},1) , epsilon );
        A = max(A,eps);
    end
    Xhat = cpdgen({A,B,C});
    Xhat(W) = 0;
    B = B .* mtkrprod(X,{A,B,C},2) ./ max( mtkrprod(Xhat,{A,B,C},2) , epsilon );
    B = max(B,eps);
    if ~onlyupdateB
        Xhat = cpdgen({A,B,C});
        Xhat(W) = 0;
        C = C .* mtkrprod(X,{A,B,C},3) ./ max( mtkrprod(Xhat,{A,B,C},3) , epsilon );
        C = max(C,eps);
    end
    
    % Display progress.
    if options.Display
        if mod(i,options.Display)==0 || i==1 || i==options.MaxIter
            curRes=norm(X(:)-Xhat(:));
            fprintf('NNTF iter. %i - res = %g\n',i,curRes/totfin);
        end
    end
    
end

end
