function [psd,f,t] = periodograms(sig,nfft,fs,win,step)

% Compute shifted signals.
n = length(sig);
idx = bsxfun(@plus,(1:nfft)',step*(0:floor((n-nfft)/step)));
idx(idx > n) = n;
t = (idx(1,:)-1)/fs;
sig = sig(idx);

% Apply window.
if nargin < 4 || isempty(win)
    win = ones(nfft,1);
end
sig = bsxfun(@times,sig,win(:));
U = win'*win;

% Compute PSD.
f = (fs/nfft)*(0:nfft-1)';
psd = fft(sig,nfft,1); % DFT
psd = psd.*conj(psd)/U; % Power estimate
if rem(nfft,2)
    select = 1:(nfft+1)/2;  % ODD
    Sxx_unscaled = psd(select,:); % Take only [0,pi] or [0,pi)
    psd = [Sxx_unscaled(1,:); 2*Sxx_unscaled(2:end,:)];  % Only DC is a unique point and doesn't get doubled
else
    select = 1:nfft/2+1;    % EVEN
    Sxx_unscaled = psd(select,:); % Take only [0,pi] or [0,pi)
    psd = [Sxx_unscaled(1,:); 2*Sxx_unscaled(2:end-1,:); Sxx_unscaled(end,:)]; % Don't double unique Nyquist point
end
f = f(select);
psd = psd/fs; % PSD estmate

end
