function ann = predictann(B,signalred,tspec)

% Compute probabilities based on LS-SVM models.
prob = zeros(1,length(tspec));
lookback = 10;
lookahead = 10;
% for i = 0:lookback-1
%     X = features(B,i,lookahead);
%     p = lssvm(X(1,:),i,lookahead);
%     prob(i+1) = p;
% end
X = features(B,signalred,lookback,lookahead);
p = lssvm(X,lookback,lookahead);
prob(1+lookback:end-lookahead) = p;
% for i = 0:lookahead-1
%     X = features(B,lookback,i);
%     p = lssvm(X(end,:),lookback,i);
%     prob(end-i) = p;
% end

plt = 0; % Set to 1 for a diagnostic plot.
if plt
    cr=gcf;
    figure(99);
    plot(prob,'o-');
    hold on;
    plot([1 length(prob)],[0 0],'r');
    ii = find(prob > 0);
end

% Filter probabilities to obtain classification.
ann = [];
wndw = 75;
step = 10;
fs = 250;
mindist = 79/1000;
tann = tspec(prob > 0); tann(end+1) = inf;
first = 1;
last = 1;
for i = 2:length(tann)
    if tann(i)-tann(last) <= 2*step/fs+1e3*eps % Don't cut off interval too early.
        last = i;
    else
        if tann(last)-tann(first) >= 4*step/fs-1e3*eps % Prevent false negatives.
            ann(end+1) = 0.5*(tann(first)+tann(last))+(wndw/fs)/2;
            if plt
                plot([ii(first);ii(last)],[1;1],'m-s');
            end
        end
        first = i;
        last = i;
    end
end

if plt
    ylim([-30 30])
    hold off;
    figure(cr);
end

% Add first and last beats using Pan-Tompkins on ECG
ann = pantomBeginEnd(ann,signalred);

% Apply second filter.
%ann = hrfilter(ann');
%ann = ann';

end

function p = lssvm(X,lb,la)

X = normalizefeatures(X,lb,la);
load(['lssvm-' int2str(lb) '-' int2str(la)]);
features = AFEm(Xsubs,'RBF_kernel',sig2,X);
p = features*w(1:size(features,2))+b;
%p = 1./(1+exp(-p));

end

function ann = pantomBeginEnd(ann,signalred)

ecg = signalred(1,:)';

% Take first and last 10 seconds of ECG data
ecgFirst = ecg(1:min(2500,size(ecg,1)));
ecgLast = ecg(max(1,size(ecg,1)-2499):end);

% Resample from 250 to 200 Hz
ecgFirst = (interp1(1/250:1/250:length(ecgFirst)/250,ecgFirst,1/200:1/200:length(ecgFirst)/250,'pchip'))';
ecgLast = (interp1(1/250:1/250:length(ecgLast)/250,ecgLast,1/200:1/200:length(ecgLast)/250,'pchip'))';

% Detrend
ecgFirst = detrendmedian(ecgFirst,5*200,40,10);
ecgLast = detrendmedian(ecgLast,5*200,40,10);

% Add 1 second of zeros to front of ecgFirst and back of ecgLast
ecgFirst = [zeros(200,1);ecgFirst];
ecgLast = [ecgLast;zeros(200,1)];

% Detect heart beat positions using Pan-Tompkins
[HRtimesFirst] = pantom(ecgFirst);
[HRtimesLast] = pantom(ecgLast);

% Transform HRtimesFirst and HRtimesLast to match atr annotations
if (HRtimesFirst(1) > 0.85 && HRtimesFirst(1) < 1), HRtimesFirst(1) = 1; end
HRtimesFirst = HRtimesFirst(HRtimesFirst>1)-1;
HRtimesLast = HRtimesLast(HRtimesLast<10)+(length(ecg)-1)/250-10;

% Merge result of pantom with previous ann
for i = 1:length(HRtimesFirst)
    if (HRtimesFirst(i) < 1 && min(abs(ann-HRtimesFirst(i))) > 0.150)
        loc = find(ann > HRtimesFirst(i),1,'first');
        ann = [ann(1:loc-1),HRtimesFirst(i),ann(loc:end)];
    end
end

for i = 1:length(HRtimesLast)
    if (HRtimesLast(i) > (length(ecg)-1)/250-1 && min(abs(ann-HRtimesLast(i))) > 0.150)
        loc = find(ann < HRtimesLast(i),1,'last');
        ann = [ann(1:loc),HRtimesLast(i),ann(loc+1:end)];
    end
end

end
