function [spec,tspec,fs,tann,f,signalred] = readrecord(recordName)

% Read record.
siginfo = wfdbdesc(recordName);
[tsig,signal] = rdsamp(recordName);

% Resample to 250 Hz.
fstarget = 250;
[signal,fs] = resamplesignals(tsig,signal,siginfo,fstarget);

% Extract independent components.
maxiter = 100;
iter = inf;
while iter >= maxiter
    [ic,~,~,iter] = fastica(signal,'numOfIC',min(size(signal,1),5), ...
        'g','gauss','approach','symm','maxNumIterations',maxiter);
end

% Extract ECG from independent components, BP/PPG and SV from signal.
des = squeeze(struct2cell(siginfo)); des = reshape(des(5,:),[],1);
signalred = classifysignals(ic,signal,des);

% Normalize signals.
signalred = normalizesignals(signalred);

% Compute spectograms.
wndw = 75;
step = 10;
frng = [3 50];
[spec,f,t] = computespecgrams(signalred,fstarget,wndw,step,frng);
tspec = t';

% Also read annotations if requested.
if nargout > 3
    if exist([recordName '.atr'],'file')
        [tann,type,subtype,chan,num] = rdann(recordName,'atr');
        tann = (tann-1)'/fs;
    else
        tann = [];
    end
end

end
